; "Ibzed"
; ZX Spectrum 48K, tiny 32b intro
; code - .ded^RMDA
; 16.07.2022, Samara, Russia

;---[Story]-----------------------------------------------------
; This is a remix of original 43b title from Kakos Nonos (2013)
; called "Ibze Megademo". Download and check original at ZXArt:
; https://zxart.ee/eng/software/demoscene/intro/64b-intro/ibze-megademo/
;
; Original product was Pentagon only and did not utilize IM2 table.
; Keeping this in mind, after byte-by-byte it was shrunken down to 24-25b.
; Some visual changes were implemented but these two works are comparable.
; Final build was targeted at 32b with some enhancements.
; Interesting fact. Original IBZE works different in Unreal TSL and Speculator 8.0 - different color scheme. Seems that Speculator fail in exact R emulation.

;---[Some tech info]--------------------------------------------
; Duration 120 seconds then RESET
; Fixed only for Pentagon 128 with 224T per one screen line
; IM2 enabled but without vector table so only for stable bus
; Several opcodes reused twice so do not try to change ORG add/remove any opcodes before you understand why they are placed here!
; Have fun with debugger!

; code footprint:
; 09b - init SP and IM2
; 15b - main FX loop
; 08b - final delays, screen FX and loop

;---[How to build]----------------------------------------------
; in SjASMplus https://github.com/z00m128/sjasmplus
; sjasmplus.exe ibzed.asm

; in Pasmo https://pasmo.speccy.org
; pasmo.exe --bin ibzed.asm ibzed.bin

; .ded

	DEVICE	ZXSPECTRUM48

	org	#fbe8
begin:
;	di
	ld	hl,#7ffb
	ld	sp,hl
	ld	a,l
	ld	i,a
	im	2		; FX + opcode reuse: LD E,(HL)
;-------IM2 handler---------------------------------------------
frame:	ld	a,r
	out	(#fe),a
	ld	a,7		; not just a border color but
				; also DELAY for LINE loop via
				; opcode reuse: CP #29
line2:	out	(#fe),a

;-------one line delay------------------------------------------
	add	hl,hl		; DELAY + HL initialization
line:	dec	a
	jr	nz,line2+1
	dec	hl		; DELAY + visual for PUSH HL
				; 166T of total delay after OUT
;---------------------------------------------------------------
	djnz	frame-1		; !!! two bytes here reused
	ei			; !!! as IM2 handler address
;---------------------------------------------------------------
; final delays, opcode shift, R alignment, bottom border + FX
	out	(#fe),a
	push	af
	inc	sp
	push	hl
	push	hl
        halt			; loop everything on FRAME-1
;---------------------------------------------------------------
finish:	
	SAVESNA	"ibzed.sna", begin
	SAVEBIN	"ibzed.bin", begin, finish-begin
;	EMPTYTRD "ibzed.trd","Ibzed"
;	SAVETRD	"ibzed.trd","ibzed.C",begin,finish-begin
;	EMPTYTAP "ibzed.tap"
;	SAVETAP "ibzed.tap",CODE,"ibzed",begin,finish-begin

;-------=RMDA=---------------------------------------------[eof]